SystemOrganization addCategory: #GeoDSL!
SystemOrganization addCategory: 'GeoDSL-Examples'!
SystemOrganization addCategory: 'GeoDSL-Tests'!

PPCompositeParser subclass: #GeoDSLGrammar
	instanceVariableNames: 'integer point pointArray anyObject poly color polygon polyline rectangle ellipse object graphics size location bearing label labelString comment sc id newMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GeoDSL'!

!GeoDSLGrammar commentStamp: '<historical>' prior: 0!
A GeoDSLGrammar for PetitParser parses a GeoSpec.

A GeoSpec starts with: 'displayIn:' followed by
- a name of the graphics 
- the screen size of the display and
- arbitrary many graphics objects.

Graphics objects are
- rectangle
- ellipse 
- polygon 
- polyline and
- newMorph (for which a Morph class must be specified)
!

GeoDSLGrammar subclass: #GeoDSLCompiler
	instanceVariableNames: ''
	classVariableNames: 'DrawingClass'
	poolDictionaries: ''
	category: 'GeoDSL'!

!GeoDSLCompiler commentStamp: '<historical>' prior: 0!
A GeoDSLCompiler for PetitParser
!

!GeoDSLCompiler class methodsFor: 'class initialization' stamp: 'hk 8/25/2014 17:59'!
drawingClass
	^DrawingClass! !

!GeoDSLCompiler class methodsFor: 'class initialization' stamp: 'hk 8/25/2014 17:57'!
drawingClass: aClass 
	DrawingClass := aClass! !

!GeoDSLCompiler methodsFor: 'productions' stamp: 'hk 2/13/2014 12:19'!
anyObject
	^ super anyObject ==> [:n | n  at: 2 ] 
		
! !

!GeoDSLCompiler methodsFor: 'productions' stamp: 'hk 3/6/2014 16:00'!
bearing
	"in radians, 0 is default"
	^ super bearing ==> [:n | n ifNil: [0 ] ifNotNil: [(n at: 2) inputValue asNumber degreesToRadians] ]
		
! !

!GeoDSLCompiler methodsFor: 'productions' stamp: 'hk 7/7/2014 16:23'!
color
	"default is empty string"
	^ super color ==> [:n | n ifNil: [ '' ] ifNotNil: [(n at: 2) inputValue ] ]
		
! !

!GeoDSLCompiler methodsFor: 'productions' stamp: 'hk 3/6/2014 15:47'!
ellipse
	^ super ellipse ==> [:n | (n at: 2) morphClass: #EllipseMorph]! !

!GeoDSLCompiler methodsFor: 'productions' stamp: 'hk 8/25/2014 18:00'!
graphics
	"create the drawing. At least one object is required to be able to calculate zoom & translation"
	^ super graphics
		==> [:n | | r  | 
			(n at: 4) ifEmpty:  [self error: 'no objects defined' ].
			 r := DrawingClass new: (n at: 2) inputValue.
			(n at: 4) do: [ :e | r  addToObjectDict:  e ]. 
			r createAllMorphsWithin: (n at: 3). 
			r ] 
		
! !

!GeoDSLCompiler methodsFor: 'productions' stamp: 'hk 3/6/2014 15:56'!
label
	"default is empty string"
	^ super label ==> [:n | n ifNil: [ '' ] ifNotNil: [n at: 2 ] ]
		
! !

!GeoDSLCompiler methodsFor: 'productions' stamp: 'hk 3/6/2014 15:56'!
labelString
	"a hack to be able to position the label string horizontally"
	^ super labelString ==> [ :n | n inputValue copyReplaceAll: '_' with: ' ' ]
		
! !

!GeoDSLCompiler methodsFor: 'productions' stamp: 'hk 1/15/2014 11:54'!
location
	^ super location ==> [:n | n at: 2] 
		
! !

!GeoDSLCompiler methodsFor: 'productions' stamp: 'hk 3/6/2014 16:21'!
newMorph
	"for existing or new morphs; check if the specified morph class exists"
	^ super newMorph 
		==> [:n | | cl |
				cl := (n at: 2) inputValue asSymbol.   
				((Smalltalk at: cl)  isKindOf: Morph class ) 
					ifFalse:  [self error: cl, ' is no morph class at ' , (n at: 1) stop printString]
					ifTrue:  [ (n at: 3) morphClass: cl; yourself] ]! !

!GeoDSLCompiler methodsFor: 'productions' stamp: 'hk 7/7/2014 15:50'!
object
	^ super object
		==> [:n | (GeoDSLObject new) 
					name: (n at: 1) inputValue;
					size: (n at: 2); 
					location: (n at: 3);
					bearing: (n at: 4);
					label: (n at: 5);
					color: (n at: 6) ]
		
! !

!GeoDSLCompiler methodsFor: 'productions' stamp: 'hk 1/15/2014 12:13'!
point
	^ super point ==> [:n | Point x: (n at: 1) inputValue asInteger y: (n at: 3) inputValue asInteger]! !

!GeoDSLCompiler methodsFor: 'productions' stamp: 'hk 9/8/2014 14:41'!
pointArray
	^ super pointArray
		==> [:n | |a s | 	s := (n at: 3) size.
						a := Array new: s + 1 .
						a at: 1 put: (n at: 2).
						a replaceFrom: 2 to: s+1 with: (n at: 3).
						a]  
		
	! !

!GeoDSLCompiler methodsFor: 'productions' stamp: 'hk 7/7/2014 15:51'!
poly
	^ super poly
		==> [:n | (GeoDSLPathshape vertices: (n at: 2)) 
					name: (n at: 1) inputValue;
					bearing: (n at: 3);
					label: (n at: 4);
					color: (n at: 5) ]! !

!GeoDSLCompiler methodsFor: 'productions' stamp: 'hk 3/6/2014 15:48'!
polygon
	^ super polygon ==> [:n | (n at: 2) morphClass: #PolygonMorph; closed: true]! !

!GeoDSLCompiler methodsFor: 'productions' stamp: 'hk 3/6/2014 15:48'!
polyline
	^ super polyline ==> [:n | (n at: 2) morphClass: #PolygonMorph; closed: false]! !

!GeoDSLCompiler methodsFor: 'productions' stamp: 'hk 3/6/2014 15:48'!
rectangle
	^ super rectangle ==> [:n | (n at: 2) morphClass: #BorderedMorph]! !

!GeoDSLCompiler methodsFor: 'productions' stamp: 'hk 3/6/2014 15:49'!
size
	"default 100@100"
	^ super size ==> [:n | n ifNil: [100@100 ] ifNotNil: [n at: 2] ]  ! !

!GeoDSLGrammar class methodsFor: 'parsing' stamp: 'hk 1/14/2014 12:05'!
compile:  aSpec
	^self parse: aSpec onError: [:msg :pos | self error: (msg, ' at: ', pos printString)]! !

!GeoDSLGrammar methodsFor: 'rules' stamp: 'hk 3/6/2014 16:20'!
anyObject
	^ ($( asParser trim:sc) , (polygon / polyline / rectangle / ellipse / newMorph) , ($) asParser trim:sc) ! !

!GeoDSLGrammar methodsFor: 'rules' stamp: 'hk 2/17/2014 21:13'!
bearing
	^ ( 'bearing:' asParser, integer ) optional trim:sc! !

!GeoDSLGrammar methodsFor: 'grammar' stamp: 'hk 7/7/2014 16:23'!
color
	^ ( 'color:' asParser , id ) optional  trim:sc! !

!GeoDSLGrammar methodsFor: 'basic rules' stamp: 'hk 2/17/2014 21:13'!
comment
 	^ $" asParser , (PPPredicateObjectParser anyExceptAnyOf: (String with: $") ) star , $" asParser! !

!GeoDSLGrammar methodsFor: 'rules' stamp: 'hk 3/6/2014 13:36'!
ellipse
	^ 'ellipse:' asParser , object! !

!GeoDSLGrammar methodsFor: 'rules' stamp: 'hk 3/6/2014 16:04'!
graphics
	^ ('displayIn:' asParser trim:sc) , id , size, anyObject star! !

!GeoDSLGrammar methodsFor: 'basic rules' stamp: 'hk 3/6/2014 16:03'!
id
	^ (#letter asParser , #word asParser star) token trim:sc ! !

!GeoDSLGrammar methodsFor: 'basic rules' stamp: 'hk 2/17/2014 21:13'!
integer
	^ ($- asParser optional , #digit asParser plus) token trim:sc! !

!GeoDSLGrammar methodsFor: 'rules' stamp: 'hk 2/17/2014 21:12'!
label
	^ ( 'label:' asParser , labelString ) optional trim:sc! !

!GeoDSLGrammar methodsFor: 'rules' stamp: 'hk 3/6/2014 16:09'!
labelString
	^ ( (#letter asParser | #digit asParser | $_ asParser) star) token trim:sc! !

!GeoDSLGrammar methodsFor: 'rules' stamp: 'hk 2/17/2014 21:12'!
location
	^ ('at:' asParser , point) trim:sc ! !

!GeoDSLGrammar methodsFor: 'rules' stamp: 'hk 3/6/2014 16:20'!
newMorph
	^ 'newMorph:' asParser , id, object! !

!GeoDSLGrammar methodsFor: 'grammar' stamp: 'hk 7/7/2014 15:47'!
object
	^ id ,size ,  location , bearing , label , color! !

!GeoDSLGrammar methodsFor: 'rules' stamp: 'hk 1/15/2014 18:02'!
point
	^ integer, $@ asParser, integer ! !

!GeoDSLGrammar methodsFor: 'rules' stamp: 'hk 2/17/2014 21:12'!
pointArray
	"contains at least 2 points for polylines and polygons"
	^ ($[ asParser, point, point plus, $] asParser ) trim:sc! !

!GeoDSLGrammar methodsFor: 'rules' stamp: 'hk 7/7/2014 15:47'!
poly
	"PolygonMorph will be used for polygons as well as polylines"
	^ id, pointArray, bearing, label, color! !

!GeoDSLGrammar methodsFor: 'rules' stamp: 'hk 1/16/2014 10:33'!
polygon
	^ 'polygon:' asParser, poly! !

!GeoDSLGrammar methodsFor: 'rules' stamp: 'hk 1/16/2014 10:34'!
polyline
	^ 'polyline:' asParser, poly! !

!GeoDSLGrammar methodsFor: 'rules' stamp: 'hk 3/6/2014 13:36'!
rectangle
	^ 'rectangle:' asParser , object! !

!GeoDSLGrammar methodsFor: 'basic rules' stamp: 'hk 2/17/2014 21:01'!
sc
	"parse whitespace and comments"

 	^#space asParser | comment! !

!GeoDSLGrammar methodsFor: 'rules' stamp: 'hk 2/17/2014 21:12'!
size
	^  (('size:' asParser, point) optional) trim:sc! !

!GeoDSLGrammar methodsFor: 'rules' stamp: 'hk 2/16/2014 12:16'!
start
	^ graphics end! !

BorderedMorph subclass: #GeoDSLCar
	instanceVariableNames: 'model'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GeoDSL-Examples'!

!GeoDSLCar commentStamp: '<historical>' prior: 0!
A GeoDSLCar is a Morph.

Any GeoDSL morph must be created using Morph>>newBounds:
The private morph method privateBounds: must be complemented to construct the morph



!

!GeoDSLCar class methodsFor: 'initialize-release' stamp: 'hk 5/14/2014 17:28'!
initialize 
	"self initialize"
	self defaultYellowButtonMenuEnabled: true! !

!GeoDSLCar methodsFor: 'menus' stamp: 'hk 5/14/2014 16:37'!
handlesMouseDown: anEvent 
		^ true. ! !

!GeoDSLCar methodsFor: 'initialize-release' stamp: 'hk 1/26/2014 13:14'!
initializeMorph
	"construct a car"
	| wheel |
	self borderWidth: 2.
	self color: Color red.  "Ferrari"
	self useRoundedCorners.
	wheel := Rectangle origin: self bounds origin extent: self bounds extent / 7.
	self addMorph: ((Morph newBounds: wheel) center: self bounds origin + (3@0 * self width / 14); color: Color black).
	self addMorph: ((Morph newBounds: wheel) center: self bounds origin + (11@0 * self width / 14); color: Color black).
	self addMorph: ((Morph newBounds: wheel) center: self bounds corner - (3@0 * self width / 14); color: Color black).
	self addMorph: ((Morph newBounds: wheel) center: self bounds corner - (11@0 * self width / 14); color: Color black).! !

!GeoDSLCar methodsFor: 'accessing' stamp: 'hk 5/22/2014 15:05'!
model: aModel 
	model := aModel! !

!GeoDSLCar methodsFor: 'menus' stamp: 'hk 5/22/2014 15:49'!
mouseDown: evt 
	"Handle a mouse down event. "
	
	| action |
	evt redButtonPressed ifTrue: [ self inspect ].
	evt yellowButtonPressed ifTrue: 
		[ action := UIManager default 
			chooseFrom: #( 'stop' 'go'  'go slow'  'inspect' )	
			values: #( stop initializeVelocity slowDown inspect)
			lines: #(  2 )
			message: 'What shall this car do?'.
		action ifNotNil:  [model perform: action]
		]! !

!GeoDSLCar methodsFor: 'private' stamp: 'hk 1/27/2014 16:37'!
privateBounds: boundsRect
	"bounds cannot be set at initialization time, thus we must intercept this morph method"
	super privateBounds: boundsRect.
	self initializeMorph
! !

Point subclass: #RoadPoint
	instanceVariableNames: 'bearing index speedlimit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GeoDSL-Examples'!

!RoadPoint commentStamp: '<historical>' prior: 0!
A NextPoint is a point and a direction (bearing).

Instance Variables
	bearing:		<Number>		- of the next point (index + 1) - in radians
	index: 			<Integer>		- of the reference point of the polygon array
	speedlimit:		<Integer>		- 0 if none
				
Note:
this bearing is in radians, not degrees.
the bearing of a point is its theta negated, it is counter-clockwise .!

!RoadPoint class methodsFor: 'as yet unclassified' stamp: 'hk 1/30/2014 17:35'!
from: aPoint at: index bearing:  radians
	^self from: aPoint at: index bearing:  radians speedlimit: 0! !

!RoadPoint class methodsFor: 'as yet unclassified' stamp: 'hk 1/30/2014 17:35'!
from: aPoint at: index bearing:  radians speedlimit: aNumber
	^(self x: aPoint x y: aPoint y)
		index: index;
		bearing: radians;
		speedlimit: aNumber ! !

!RoadPoint methodsFor: 'accessing' stamp: 'hk 1/28/2014 13:21'!
bearing
	^ bearing! !

!RoadPoint methodsFor: 'accessing' stamp: 'hk 1/28/2014 13:22'!
bearing: aRadian
	bearing := aRadian! !

!RoadPoint methodsFor: 'accessing' stamp: 'hk 1/28/2014 13:37'!
index
	^ index! !

!RoadPoint methodsFor: 'accessing' stamp: 'hk 1/28/2014 13:37'!
index: anInteger
	index := anInteger! !

!RoadPoint methodsFor: 'accessing' stamp: 'hk 1/31/2014 07:14'!
speedlimit
	^ speedlimit > 0 ifTrue: [ speedlimit ] ifFalse: [ 300 ]! !

!RoadPoint methodsFor: 'accessing' stamp: 'hk 1/30/2014 16:59'!
speedlimit: anInteger
	speedlimit := anInteger! !

PPCompositeParserTest subclass: #GeoDSLParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GeoDSL-Tests'!

!GeoDSLParserTest commentStamp: '<historical>' prior: 0!
A GeoDSLParserTest tests the grammar itself.

!

!GeoDSLParserTest methodsFor: 'accessing' stamp: 'hk 3/5/2014 13:47'!
parserClass
	^GeoDSLGrammar ! !

!GeoDSLParserTest methodsFor: 'accessing' stamp: 'hk 3/5/2014 13:51'!
testBearing
	self parse:  '  bearing: 27 '  rule: #bearing! !

!GeoDSLParserTest methodsFor: 'accessing' stamp: 'hk 3/5/2014 13:59'!
testComment
	self parse:  '"a comment"'  rule: #comment! !

!GeoDSLParserTest methodsFor: 'accessing' stamp: 'hk 3/6/2014 15:06'!
testEllipse
	self parse:  'ellipse: lane2 size: 60@20 at: 375@100 label: spur2'  rule: #ellipse! !

!GeoDSLParserTest methodsFor: 'accessing' stamp: 'hk 3/6/2014 15:02'!
testGraphics
	self parse:  'displayIn: myGraphics size: 1000@500 (rectangle: lane1 size: 60@20 at: 300@100 bearing: 90)'  rule: #graphics! !

!GeoDSLParserTest methodsFor: 'accessing' stamp: 'hk 3/5/2014 13:52'!
testInteger
	self parse:  '  -27 '  rule: #integer! !

!GeoDSLParserTest methodsFor: 'accessing' stamp: 'hk 3/5/2014 15:00'!
testLabel
	self parse:  ' label: ___a123_b24 '  rule: #label  ! !

!GeoDSLParserTest methodsFor: 'accessing' stamp: 'hk 3/5/2014 13:54'!
testLocation
	self parse:  ' at: -34 @ -13 '  rule: #location! !

!GeoDSLParserTest methodsFor: 'accessing' stamp: 'hk 3/6/2014 16:21'!
testNewType
	self parse:  'newMorph: CircleMorph  lane2 size: 60@20 at: 375@100'  rule: #newMorph! !

!GeoDSLParserTest methodsFor: 'accessing' stamp: 'hk 3/6/2014 15:05'!
testObject
	self parse:  ' lane1 size: 60@20 at: 300@100 bearing: 90 label: spur1'  rule: #object! !

!GeoDSLParserTest methodsFor: 'accessing' stamp: 'hk 3/5/2014 13:56'!
testPointArray
	self parse:  ' [ 3@4  5@7] '  rule: #pointArray! !

!GeoDSLParserTest methodsFor: 'accessing' stamp: 'hk 3/6/2014 15:10'!
testPoly
	self parse:  'bla [200@200 250@220 300@250 230@270] label: border'  rule: #poly! !

!GeoDSLParserTest methodsFor: 'accessing' stamp: 'hk 3/5/2014 14:05'!
testPolygon1
	self parse:  'polygon: bla [200@200 250@220 300@250 230@270] label: border'  rule: #polygon! !

!GeoDSLParserTest methodsFor: 'accessing' stamp: 'hk 3/5/2014 14:05'!
testPolygon2
	self parse:  'polygon: bla [200@200 250@220 300@250 230@270] '  rule: #polygon! !

!GeoDSLParserTest methodsFor: 'accessing' stamp: 'hk 3/5/2014 14:06'!
testPolyline
	self parse:  'polyline: blb [200@300 250@320 300@350 230@370] '  rule: #polyline! !

!GeoDSLParserTest methodsFor: 'accessing' stamp: 'hk 3/6/2014 15:05'!
testRectangle
	self parse:  'rectangle: lane1 size: 60@20 at: 300@100 bearing: 90 label: spur1'  rule: #rectangle! !

!GeoDSLParserTest methodsFor: 'accessing' stamp: 'hk 3/5/2014 14:01'!
testSize
	self parse:  ' size: 5@6 '  rule: #size! !

!GeoDSLParserTest methodsFor: 'accessing' stamp: 'hk 3/6/2014 16:21'!
testWrongType
	"MyNotMorph can be parsed"
	self parse:  'newMorph: MyNotMorph  lane2 size: 60@20 at: 375@100'  rule: #newMorph! !

!GeoDSLParserTest methodsFor: 'accessing' stamp: 'hk 3/6/2014 15:42'!
testWrongType2
	"MyNotMorph can not be used"
	self fail:  ' (MyNotMorph  lane2 size: 60@20 at: 375@100) '  rule: #anyObject 
! !

GeoDSLParserTest subclass: #GeoDSLProductionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GeoDSL-Tests'!

!GeoDSLProductionTest commentStamp: '<historical>' prior: 0!
A GeoDSLProductionTest tests the interconnection of our domain model made by the DSL text. 


!

GeoDSLProductionTest subclass: #GeoDSLCompilerTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GeoDSL-Tests'!

!GeoDSLCompilerTests commentStamp: '<historical>' prior: 0!
A GeoDSLCompilerTests tests various complete specs with and withour errors.

These are kind of system tests


!

!GeoDSLCompilerTests class methodsFor: 'example texts DSL' stamp: 'hk 7/7/2014 15:58'!
allMorphsSpec

	^'displayIn: myGraphics size: 1000@500    											
		(rectangle: lane1 size: 60@20 at: 300@100 bearing: 90 label: spur1)   
	   	(rectangle: lane2 size: 60@20 at: 375@100 label: spur2  color: yellow)				
		
		(rectangle: agv1 size: 22@10 at: 500@150 bearing: 10 label: agv1)   	
		(rectangle: agv2 size: 22@10 at: 500@180 bearing: 60 label: agv2)   
		
		(ellipse: c1 size: 200@200 at: 400@300 bearing: 1 label: the_circle)   
		(newMorph: GeoDSLCar a1 size: 32@10 at: 500@300 bearing: 90 label: car1)   			
		(ellipse: c2 size:  200@100 at: 400@300 bearing: 135 label: the_ellipse color: red) 
		
		(polygon: bla [200@200 250@220 300@250 230@270] label: border)
		(polyline: blb [200@300 250@320 300@350 230@370] label: line color: blue)'! !

!GeoDSLCompilerTests class methodsFor: 'example texts DSL' stamp: 'hk 3/6/2014 15:23'!
allMorphsSpecWithComments

	^'  "GeoDSLExample new createFrom: self geoSpec" 
	displayIn: myGraphics size: 1000@500    													"defines the initial zoom"
		(rectangle: lane1 size: 60@20 at: 300@100 bearing: 90 label: spur1)     				 "first object" 
	   	(rectangle: lane2 size: 60@20 at: 375@100 label: spur2)				
		
		(rectangle: "even here you my place a comment" agv1 size: 22@10 at: 500@150 bearing: 10 label: agv1)   	
		(rectangle: agv2 size: 22@10 at: 500@180 bearing: 60 label: agv2)   		 			"fourth object"
		
		(ellipse: c1 size: 200@200 at: 400@ "bad place for a comment" 300 bearing: 1 label: the_circle)   
		(newMorph: GeoDSLCar a1 size: 32@10 at: 500@300 bearing: 90 label: car1)     		"sixth object"			
		(ellipse: c2 size:  200@100 at: 400@300 bearing: 135 label: the_ellipse) 
		
		(polygon: bla [200@200 250@220 300@250 230@270] label: border)
		(polyline: blb [200@300 250@320 300@350 230@370] label: line)
		
				"another comment"

		'! !

!GeoDSLCompilerTests class methodsFor: 'example texts DSL' stamp: 'hk 3/6/2014 15:23'!
borderedMorphSpec

	^'displayIn: myGraphics size: 1000@500 (rectangle: lane1 size: 60@20 at: 300@100 bearing: 90 label: spur1)'! !

!GeoDSLCompilerTests class methodsFor: 'example texts DSL' stamp: 'hk 3/6/2014 15:23'!
borderedMorphsSpec

	^'displayIn: myGraphics size: 1000@500 (rectangle: lane1 size: 60@20 at: 300@100 bearing: 90 label: spur1) (rectangle: lane2 size: 60@20 at: 375@100 label: spur2) (rectangle: agv1 size: 22@10 at: 500@150 bearing: 10 label: agv1) (rectangle: agv2 size: 22@10 at: 500@180 bearing: 60 label: agv2)'! !

!GeoDSLCompilerTests class methodsFor: 'example texts DSL' stamp: 'hk 3/6/2014 15:23'!
borderedMorphsWithSpaceSpec

	^' displayIn:  myGraphics  size:  1000   @  500 
		(rectangle:   lane1  size:   60 @ 20   at:   300@100   bearing:   90   label:   spur1  ) 
		(rectangle:   lane2 size: 60   @      20 at:      375     @     100      label: spur2) 
		(rectangle:   agv1 size: 22@10 at: 500@150 bearing: 10 label: agv1) 
		(rectangle:   agv2   size:   22@10   at:   500@180   bearing:   60   label:   agv2)'! !

!GeoDSLCompilerTests class methodsFor: 'example texts DSL' stamp: 'hk 3/6/2014 15:23'!
doubleMorphSpec

	^'displayIn: myGraphics size: 1000@500 
		(rectangle: lane1 size: 60@20 at: 300@100 bearing: 90 label: spur1)
		(rectangle: lane1 size: 60@20 at: 300@100 bearing: 90 label: spur1)		
	'! !

!GeoDSLCompilerTests class methodsFor: 'example texts DSL' stamp: 'hk 3/6/2014 15:24'!
ellipseMorphSpec

	^'displayIn: myGraphics size: 1000@500 (ellipse:  c2 size:  200@100 at: 400@300 bearing: 135 label: the_ellipse)'! !

!GeoDSLCompilerTests class methodsFor: 'example texts DSL' stamp: 'hk 2/15/2014 15:01'!
noMorphSpec

	^'displayIn: myGraphics size: 1000@500 '! !

!GeoDSLCompilerTests class methodsFor: 'example texts DSL' stamp: 'hk 2/13/2014 13:18'!
polygonMorphSpec

	^'displayIn: myGraphics size: 1000@500 (polygon: bla [200@200 250@220 300@250 230@270] label: border)'! !

!GeoDSLCompilerTests methodsFor: 'tests' stamp: 'hk 3/5/2014 15:20'!
testAllMorphs
	"multiple morphs - no space"
	self parse: (GeoDSLCompilerTests allMorphsSpec).
	self should: [result isKindOf: GeoDSLDrawing]! !

!GeoDSLCompilerTests methodsFor: 'tests' stamp: 'hk 3/5/2014 15:16'!
testAllMorphsWithComments
	"multiple morphs - no space"
	self parse: (GeoDSLCompilerTests allMorphsSpecWithComments).
	self should: [result isKindOf: GeoDSLDrawing]! !

!GeoDSLCompilerTests methodsFor: 'tests' stamp: 'hk 3/5/2014 15:21'!
testBorderedMorph
	"one morph - no space"
	self parse: (GeoDSLCompilerTests borderedMorphSpec).
	self should: [(result tMorphNamed:  'lane1' ) isKindOf: TransformationMorph ].
		self should: [(result morphNamed:  'lane1' )  isKindOf: BorderedMorph ]! !

!GeoDSLCompilerTests methodsFor: 'tests' stamp: 'hk 3/5/2014 15:16'!
testBorderedMorphs
	"multiple morphs - no space"
	self parse: (GeoDSLCompilerTests borderedMorphsSpec).
	self should: [result isKindOf: GeoDSLDrawing]! !

!GeoDSLCompilerTests methodsFor: 'tests' stamp: 'hk 3/5/2014 15:16'!
testBorderedMorphsWithSpace
	"multiple morphs - no space"
	self parse: (GeoDSLCompilerTests borderedMorphsWithSpaceSpec).
	self should: [result isKindOf: GeoDSLDrawing]! !

!GeoDSLCompilerTests methodsFor: 'tests' stamp: 'hk 3/5/2014 15:17'!
testDoubleNames
	self should: [self parse: (GeoDSLCompilerTests doubleMorphSpec)] raise: Error! !

!GeoDSLCompilerTests methodsFor: 'tests' stamp: 'hk 3/5/2014 15:17'!
testEllipseMorph
	"one morph - no space"
	self parse:  (GeoDSLCompilerTests ellipseMorphSpec).
	self should: [result isKindOf: GeoDSLDrawing]! !

!GeoDSLCompilerTests methodsFor: 'tests' stamp: 'hk 3/5/2014 15:17'!
testFalseMorphClass
	self should: [self parse: (GeoDSLCompilerTests falseMorphSpec)] raise: Error! !

!GeoDSLCompilerTests methodsFor: 'tests' stamp: 'hk 3/5/2014 15:17'!
testPolygonMorph
	"one morph - no space"
	self parse: (GeoDSLCompilerTests polygonMorphSpec).
	self should: [result isKindOf: GeoDSLDrawing]! !

!GeoDSLCompilerTests methodsFor: 'tests' stamp: 'hk 3/5/2014 15:17'!
testnoMorph
	self should: [self parse: (GeoDSLCompilerTests noMorphSpec)] raise: Error! !

GeoDSLCompilerTests subclass: #GeoDSLInterfaceTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GeoDSL-Tests'!

!GeoDSLInterfaceTests commentStamp: '<historical>' prior: 0!
A GeoDSLInterfaceTests tests the interface (basic and convenience of our machine at GeoDSLDrawing


!

!GeoDSLInterfaceTests methodsFor: 'tests' stamp: 'hk 3/5/2014 15:18'!
testAccess
	self parse:  (GeoDSLCompilerTests borderedMorphSpec).		"get a drawing with one morph"
	self should: [(result objectNamed:  'lane1' )  isKindOf: GeoDSLObject  ].			"access object"
	self should: [(result tMorphNamed:  'lane1' ) isKindOf: TransformationMorph ].		"access tMorph "
	self should: [(result morphNamed:  'lane1' )  isKindOf: BorderedMorph ]			"access morph"

! !

!GeoDSLInterfaceTests methodsFor: 'tests' stamp: 'hk 3/5/2014 15:18'!
testAddObj
	"add a non polygon Morph "
	self parse:  (GeoDSLCompilerTests borderedMorphSpec).  "get a drawing with one morph"
	result add: #CircleMorph name: 'bla' size: 100@100 at: 0@0.
	self should: [(result morphNamed:  'bla' )  isKindOf: CircleMorph ].
! !

!GeoDSLInterfaceTests methodsFor: 'tests' stamp: 'hk 3/5/2014 15:18'!
testAddPoly
	self parse:  (GeoDSLCompilerTests borderedMorphSpec).			"get a drawing with one morph"
	result addPoly: 'bla' vertices: { 100@100  . 200@200 .  0@0} closed: true.		"add a Polygon"
	result addPoly: 'blb' vertices: { 100@100  . 200@200 .  0@0} closed: false.		"add a Polyline"
	self should: [(result objectNamed:  'bla' )  isKindOf: GeoDSLPathshape ].
	self should: [(result objectNamed:  'bla' )  closed = true].
	self should: [(result objectNamed:  'blb' )  isKindOf: GeoDSLPathshape ].
	self should: [(result objectNamed:  'blb' )  closed = false].
! !

!GeoDSLInterfaceTests methodsFor: 'tests' stamp: 'hk 3/5/2014 15:18'!
testAddRemove
	self parse: (GeoDSLCompilerTests borderedMorphSpec).		"get a drawing with one morph"
	result add: ((GeoDSLObject new)
			morphClass: #BorderedMorph;
			name: 'blc' ;
			size: 100@100;
			location: 100@100;
			bearing: 0;
			yourself).
	self should: [(result objectNamed:  'blc' )  isKindOf: GeoDSLObject  ].				"access object"
	self should: [(result tMorphNamed:  'blc' ) isKindOf: TransformationMorph ].		"access tMorph "
	self should: [(result morphNamed:  'blc' )  isKindOf: BorderedMorph ].			   	"access morph"
	result removeObjectNamed: 'blc' .
	self should:  [(result objectNamed:  'blc' )  isKindOf: GeoDSLObject] raise: Error .	"access missing object"

! !

!GeoDSLInterfaceTests methodsFor: 'tests' stamp: 'hk 3/5/2014 15:19'!
testChangeAngle
	|  d|
	self parse: (GeoDSLCompilerTests borderedMorphSpec).  "get a drawing with one morph"

	self shouldnt: [(result tMorphNamed:  'lane1' ) rotationDegrees = 10 ].	
	result change: 'lane1' at: #rotationDegrees: to: 10 .
	d := (result tMorphNamed:  'lane1' )  rotationDegrees. 
	self should: [ d = 10] description: 'expected 10, found ', d printString  .

! !

!GeoDSLInterfaceTests methodsFor: 'tests' stamp: 'hk 3/5/2014 15:19'!
testChangeColor
	|  c|
	self parse: (GeoDSLCompilerTests borderedMorphSpec).  "get a drawing with one morph"

	self shouldnt: [(result morphNamed:  'lane1' ) color = Color lightMagenta  ].	
	result change: 'lane1' at: #color: to: Color lightMagenta .
	c := (result morphNamed:  'lane1' ) color. 
	self should: [c = Color lightMagenta  ] description: 'light magenta expected, found ', c printString .
! !

!GeoDSLInterfaceTests methodsFor: 'tests' stamp: 'hk 3/5/2014 15:19'!
testMoveBy
	|  b p |
	self parse:  (GeoDSLCompilerTests borderedMorphSpec).  "get a drawing with one morph"

	self should: [(result objectNamed:  'lane1' ) location = (300@100)].					"bearing is 90 degrees (north)"
	result moveObject: 'lane1' by: 100 rotate: 10 .
	p := (result objectNamed:  'lane1' ) location. 
	b := (result objectNamed:  'lane1' ) bearing radiansToDegrees . 
	self should: [ p = (300@0)]  description: ' 300@0 expected, found ', p printString.
	self should: [ b = 80] description: ' 80 expected, found ', b printString.
! !

!GeoDSLInterfaceTests methodsFor: 'tests' stamp: 'hk 3/5/2014 15:19'!
testMoveTo
	|  p b |
	self parse:  (GeoDSLCompilerTests borderedMorphSpec).  "get a drawing with one morph"

	self should: [(result objectNamed:  'lane1' ) location = (300@100) ].					"bearing is 90 degrees (north)"
	result moveObject: 'lane1' To: 100@200 bearing: 10 .
	p := (result objectNamed:  'lane1' ) location. 
	b := (result objectNamed:  'lane1' ) bearing radiansToDegrees. 
	self should: [ p = (100@200) ]  description: ' 100@200 expected, found ', p printString.
	self should: [ b = 10  ] description: ' 10 expected, found ', b printString.
! !

!GeoDSLProductionTest methodsFor: 'as yet unclassified' stamp: 'hk 3/5/2014 14:36'!
parserClass
	^GeoDSLCompiler! !

!GeoDSLProductionTest methodsFor: 'as yet unclassified' stamp: 'hk 3/5/2014 14:39'!
testBearing
	"27 degreesToRadians"
	super testBearing.
	self assert: result equals:  0.47123889803846897! !

!GeoDSLProductionTest methodsFor: 'as yet unclassified' stamp: 'hk 3/6/2014 15:13'!
testEllipse
	super testEllipse.
	self assert: result morphClass equals:  #EllipseMorph! !

!GeoDSLProductionTest methodsFor: 'as yet unclassified' stamp: 'hk 3/5/2014 15:12'!
testGraphics
	super testGraphics.
	self should: (result isKindOf: GeoDSLDrawing)! !

!GeoDSLProductionTest methodsFor: 'as yet unclassified' stamp: 'hk 3/5/2014 15:00'!
testLabel
	super testLabel.
	self assert: result  equals:  '   a123 b24'! !

!GeoDSLProductionTest methodsFor: 'as yet unclassified' stamp: 'hk 3/5/2014 15:02'!
testLocation
	super testLocation.
	self assert: result  equals:  (34@13) negated! !

!GeoDSLProductionTest methodsFor: 'as yet unclassified' stamp: 'hk 3/6/2014 15:14'!
testNewType
	super testNewType.
	self assert: result morphClass equals:  #CircleMorph! !

!GeoDSLProductionTest methodsFor: 'as yet unclassified' stamp: 'hk 3/6/2014 15:09'!
testObject 
	super testObject.
	self assert: (result isKindOf:  GeoDSLObject) ! !

!GeoDSLProductionTest methodsFor: 'as yet unclassified' stamp: 'hk 3/6/2014 15:11'!
testPoly
	super testPoly.
	self assert: (result  isKindOf: GeoDSLPathshape)! !

!GeoDSLProductionTest methodsFor: 'as yet unclassified' stamp: 'hk 3/5/2014 15:04'!
testPolygon
	super testPolygon1.
	self assert: result morphClass equals:  #PolygonMorph! !

!GeoDSLProductionTest methodsFor: 'as yet unclassified' stamp: 'hk 3/6/2014 15:12'!
testRectangle
	super testRectangle.
	self assert: result morphClass equals:  #BorderedMorph! !

!GeoDSLProductionTest methodsFor: 'as yet unclassified' stamp: 'hk 3/6/2014 15:20'!
testWrongType
	"MyNotMorph can not be used"
	self should: 
	[super testWrongType]
	raise: Error ! !

Object subclass: #GeoDSLDrawing
	instanceVariableNames: 'realBounds screenBounds zoom translation objectDict canvas title'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GeoDSL'!

!GeoDSLDrawing commentStamp: '<historical>' prior: 0!
A GeoDSLDrawing holds all GeoDSLObjects.
It facilitates the coordinate transforms.

interface methods allow to add, remove and move objects (using their realWorld coordinates)

Instance Variables
	canvas:			<Morph>		- the morph that contains all other morphs
	objectDict:		<Dictionary>	- of all the named graphis objects
	realBounds:	<Rectangle>	- bounds of all objects in rela world
	screenBounds:	<Rectangle>	- bounds of all screen objects
	zoom:			<Number>		- scale of screen / realWorld
	translation:		<Object>		- of realWorld to screen (at 0@0)
	title:			<String>		- for transport from DSL-text to the window

Todo: =========================================
- createAllMorphsWithin: nicht alphabetisch sortieren, sondern nach Auftreten in der DSL 
- Tests für interface-agv!

!GeoDSLDrawing class methodsFor: 'instance creation' stamp: 'hk 1/20/2014 14:33'!
new: aString 
 	^self new initialize: aString! !

!GeoDSLDrawing methodsFor: 'inteface-basic' stamp: 'hk 2/13/2014 12:01'!
add: aGeoDSLObject
	"add a pre-fabricated GeoDSLObject"
	
	self addToObjectDict: aGeoDSLObject.
	self resetZoom.	
	canvas addMorph: (aGeoDSLObject asMorph) 
			
		! !

!GeoDSLDrawing methodsFor: 'interface-convenience' stamp: 'hk 2/15/2014 13:22'!
add: aMorphClass name: aString size: aSize at: aPoint 
	"add a non-polygon object"

	^self add: aMorphClass name: aString size: aSize at: aPoint bearing: 0 label: ''

! !

!GeoDSLDrawing methodsFor: 'interface-convenience' stamp: 'hk 2/15/2014 13:22'!
add: aMorphClass name: aString size: aSize at: aPoint bearing: aRadian label:  aString2
	"add a non-polygon object"
	
	self add:  ((GeoDSLObject new)
			morphClass: aMorphClass asSymbol;
			name: aString;
			size: aSize;
			location: aPoint;
			bearing: aRadian;
			label: aString2;
			yourself)                 
	
			
		! !

!GeoDSLDrawing methodsFor: 'interface-convenience' stamp: 'hk 2/15/2014 13:22'!
addPoly: aString vertices: anArray bearing: aRadian label:  aString2 closed: aBoolean
	"add a polygon"
	
	self add: ((GeoDSLPathshape vertices: anArray)
			morphClass: PolygonMorph name asSymbol;
			name: aString;
			bearing: aRadian;
			label: aString2;
			closed: aBoolean ;
			yourself)! !

!GeoDSLDrawing methodsFor: 'interface-convenience' stamp: 'hk 2/15/2014 13:22'!
addPoly: aString vertices: anArray closed: aBoolean
	"add a polygon"

 	self addPoly: aString vertices: anArray bearing: 0 label:  '' closed: aBoolean
		! !

!GeoDSLDrawing methodsFor: 'parsing' stamp: 'hk 2/14/2014 13:25'!
addToObjectDict: aGeoDSLObject 
	(objectDict includesKey:  aGeoDSLObject name1)
		ifTrue: [self error:  'double names in GeoDSL' ].
	objectDict add: aGeoDSLObject name1 -> aGeoDSLObject.
	self mergeRealBounds: aGeoDSLObject realBounds.
	^ aGeoDSLObject! !

!GeoDSLDrawing methodsFor: 'accessing' stamp: 'hk 1/20/2014 14:38'!
canvas
	^ canvas! !

!GeoDSLDrawing methodsFor: 'interface-convenience' stamp: 'hk 2/15/2014 13:24'!
change: aName at: aProperty to: aValue 
	"change a property of one of my GeoDSLObjects - ie the morph"
	
	|m|
	m := (self objectNamed: aName) morph.
	"all morphs are wrapped by a TransformationMorph"
	(m firstSubmorph respondsTo: aProperty) ifTrue: 
		[ ^ m firstSubmorph perform: aProperty with: aValue ].
	(m  respondsTo: aProperty) ifTrue: 
		[ m  perform: aProperty with: aValue ].
	
 	
			
		! !

!GeoDSLDrawing methodsFor: 'parsing' stamp: 'hk 7/7/2014 17:48'!
createAllMorphsWithin: aPoint 

	| submorphs |
	screenBounds := 0 @ 0 extent: aPoint.
	self resetZoom.	
	submorphs := OrderedCollection new.
	objectDict  keysSortedSafely do:  [:k  | submorphs add: (objectDict at: k) asMorph ].  "alphabetic order" 
	canvas addAllMorphs: submorphs ! !

!GeoDSLDrawing methodsFor: 'initialize-release' stamp: 'hk 1/27/2014 13:52'!
initialize: aString 
	title := aString.
	canvas := PasteUpMorph new. 
	objectDict := Dictionary new

! !

!GeoDSLDrawing methodsFor: 'private' stamp: 'hk 7/4/2014 17:06'!
mergeRealBounds: aRectangle 
	| largerRect |
	largerRect := aRectangle expandBy: 10 . 
	realBounds ifNil: [ realBounds := largerRect]
	ifNotNil: [realBounds := realBounds quickMerge: largerRect]
	! !

!GeoDSLDrawing methodsFor: 'inteface-basic' stamp: 'hk 2/13/2014 11:57'!
morphNamed: aString 
	"answer the basic Morph of the named object"

	^(self objectNamed: aString) morph firstSubmorph 		
		! !

!GeoDSLDrawing methodsFor: 'interface-convenience' stamp: 'hk 2/15/2014 13:26'!
moveObject: aString To: aPoint bearing: anAngle 
	"move the named object to aPoint and rotate to anAngle"
	
	(self objectNamed: aString) moveTo: aPoint bearing: anAngle degreesToRadians zoom: zoom! !

!GeoDSLDrawing methodsFor: 'interface-convenience' stamp: 'hk 2/15/2014 13:38'!
moveObject: aString by: aNumber rotate: anAngle 
	"move the named object by aNumber meters in its known direction and then rotate right by anAngle"

	(self objectNamed: aString) moveBy: aNumber right: anAngle degreesToRadians zoom: zoom! !

!GeoDSLDrawing methodsFor: 'inteface-basic' stamp: 'hk 7/1/2014 23:35'!
objectNamed: aString 
	"basic access"
	
	^self objectNamed: aString ifAbsent: [self error: 'object ', aString, ' not found on remove' ]! !

!GeoDSLDrawing methodsFor: 'inteface-basic' stamp: 'hk 7/1/2014 23:34'!
objectNamed: aString ifAbsent: aBlock
	"basic access"
	
	^objectDict at: aString ifAbsent: [^aBlock value]! !

!GeoDSLDrawing methodsFor: 'private' stamp: 'hk 1/26/2014 18:31'!
originalZoom
	^ (screenBounds width / realBounds width)
					min: (screenBounds height / realBounds height)
	! !

!GeoDSLDrawing methodsFor: 'inteface-basic' stamp: 'hk 2/15/2014 13:17'!
removeObjectNamed: aString 
	"remove an existing GeoDSLObject"
	
	canvas removeMorph: (self objectNamed: aString) morph.
	objectDict removeKey: aString 
			
		! !

!GeoDSLDrawing methodsFor: 'actions' stamp: 'hk 1/26/2014 17:49'!
resetZoom
	self zoom: self originalZoom! !

!GeoDSLDrawing methodsFor: 'accessing' stamp: 'hk 1/27/2014 14:50'!
screenBounds
	^ screenBounds ! !

!GeoDSLDrawing methodsFor: 'private' stamp: 'hk 1/21/2014 14:18'!
setZoomTranslation
	objectDict valuesDo: [:o | o zoom: zoom translate: translation]! !

!GeoDSLDrawing methodsFor: 'inteface-basic' stamp: 'hk 2/13/2014 11:57'!
tMorphNamed: aString 
	"answer the transformationMorph of the named object"

	^(self objectNamed: aString) morph			
		! !

!GeoDSLDrawing methodsFor: 'accessing' stamp: 'hk 1/27/2014 11:49'!
title
	^ title! !

!GeoDSLDrawing methodsFor: 'private' stamp: 'hk 1/21/2014 16:46'!
translationWith: aZoom
	^ (aZoom * realBounds origin) rounded negated  + canvas bounds origin.

		! !

!GeoDSLDrawing methodsFor: 'accessing' stamp: 'hk 1/29/2014 14:15'!
zoom
	^ zoom ! !

!GeoDSLDrawing methodsFor: 'private' stamp: 'hk 1/23/2014 13:07'!
zoom: aZoom
	zoom := aZoom.
	translation := self translationWith: aZoom.
	"zoom all my objects"
	self setZoomTranslation! !

!GeoDSLDrawing methodsFor: 'actions' stamp: 'hk 1/21/2014 16:26'!
zoomIn
	self zoom: zoom * 1.5! !

!GeoDSLDrawing methodsFor: 'actions' stamp: 'hk 1/21/2014 16:27'!
zoomOut
	self zoom: zoom * 0.5! !

Object subclass: #GeoDSLModel
	instanceVariableNames: 'drawing window'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GeoDSL'!

!GeoDSLModel commentStamp: '<historical>' prior: 0!
A GeoDSLModel works with GeoDSLGrammar and GeoDSLCompiler

Instance Variables
	drawing:		<GeoDSLDrawing>
	window:		<StandardWindow>
!

GeoDSLModel subclass: #GeoDSLExample
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GeoDSL-Examples'!

!GeoDSLExample commentStamp: '<historical>' prior: 0!
A GeoDSLExample holds som simple examples for the GeoDSL

!

!GeoDSLExample class methodsFor: 'examples' stamp: 'hk 8/18/2014 17:21'!
animation
	"self animation"
	|x|
	x := GeoDSLExample new.
	x drawingFrom: GeoDSLExample animationSpec .
	x open.
	x drawing change: 'schrank' at: #color: to: Color lightBlue   .
	"flashing"	
	[5 timesRepeat:  
		[x drawing change: 'c1' at: #color: to: Color red. 
		x drawing change: 'c2' at: #color: to: Color green. 
		(Delay forSeconds: 1) wait.
		x drawing change: 'c1' at: #color: to: Color green . 
		x drawing change: 'c2' at: #color: to: Color red. 
		(Delay forSeconds: 1) wait
		] ] fork.
! !

!GeoDSLExample class methodsFor: 'geoSpec' stamp: 'hk 3/6/2014 15:28'!
animationSpec
	^ '      																				"GeoDSLExample new createFrom: self animationSpec"
	displayIn: myGraphics size: 1000@500    											"defines the initial zoom"
		(ellipse:  c1 size: 20@20 at: 150@100 )   
		(ellipse:  c2 size: 20@20 at: 250@100 )   
		(rectangle:  schrank size: 200@400 at: 200@250   label: ___________________schaltschrank)    
		"und noch n comment"
'! !

!GeoDSLExample class methodsFor: 'examples' stamp: 'hk 8/18/2014 16:27'!
example
	"self example"
	|x|
	x := GeoDSLExample new.
	x drawingFrom: GeoDSLExample geoSpec .
	x openWithBackground: 'Nbgring.jpg' .
	"change existing objects"
	x drawing change: 'lane1' at: #color: to: Color lightBlue .
	x drawing change: 'lane2' at: #color: to: Color lightGreen  .
	x drawing change: 'lane2' at: #rotationDegrees: to: 10 .
	x drawing change: 'agv1' at: #color: to: Color lightBlue .
	x drawing change: 'agv2' at: #color: to: Color yellow  .
	"add an object"
	x drawing add: #GeoDSLCar  name: 'test1' size: 66@20 at: 100@200 bearing: 90 label: 'my Ferrari'. 
	x drawing change: 'test1' at: #rotationDegrees: to: 45 .
	"animate"
	[35 timesRepeat: [ (Delay forMilliseconds: 20) wait.  x drawing  moveObject: 'test1' by:  2 rotate: 1 ]] fork.
	[360 timesRepeat: [ (Delay forMilliseconds: 10) wait.  x drawing  moveObject: 'a1' by:  1.75 rotate: -1 ]] fork.
	"remove an object"
	[ (Delay forSeconds: 4 )wait.  x drawing removeObjectNamed: 'test1' ] fork! !

!GeoDSLExample class methodsFor: 'examples' stamp: 'hk 2/14/2014 16:11'!
exampleTrafficSimulation
	"GeoDSLExample exampleTrafficSimulation"
	TrafficSimulation newOn: TSRoad road1 with: 10.
	
"	X  := TrafficSimulation newOn: TSRoad road1 with: 7
	[X     next] timeToRun
	MessageTally spyOn: [100 timesRepeat: [ X next ]] 

"! !

!GeoDSLExample class methodsFor: 'geoSpec' stamp: 'hk 3/6/2014 15:28'!
geoSpec
	^ '      																				"GeoDSLExample new createFrom: self geoSpec"
	displayIn: myGraphics size: 1000@500    											"defines the initial zoom"
		(rectangle:  lane1 size: 60@20 at: 300@100  bearing: 90   label: spur1)    "first object"
	   	(rectangle:  lane2 size: 60@20 at: 375@100 label: spur2 )				"second object"
		
		(rectangle:  agv1 size: 22@10 at: 500@150 bearing:  10 label: agv1)   	"third object"
		(rectangle:  agv2 size: 22@10 at: 500@180 bearing: 60 label: agv2 )   	"fourth object"
		
		(ellipse:  c1 size: 200@200 at: 400@300 bearing: 1  label: the_circle)   	"fifth object"
		(newMorph: GeoDSLCar  a1 size: 32@10 at: 500@300 bearing: 90  label: car1)   			"sixth object"
		(ellipse:  c2 size:  200@100 at: 400@300 bearing: 135 label: the_ellipse ) 
		
		(polygon: bla [200@200 250@220 300@250 230@270] label: border )
		(polyline: blb [200@300 250@320 300@350 230@370] label: line )		
'! !

!GeoDSLExample class methodsFor: 'geoSpec' stamp: 'hk 3/6/2014 15:59'!
geoSpec1
		"GeoDSLExample new createFrom: self geoSpec1"
^' displayIn: myGraphics size: 1100@550
		(rectangle:  corner1 size: 10@10 at: 4700@900 )	
'! !

!GeoDSLExample class methodsFor: 'geoSpec' stamp: 'hk 3/6/2014 15:27'!
geoSpecTrafficSimulation
		"GeoDSLExample new createFrom: self geoSpecTrafficSimulation"
^' displayIn: myGraphics size: 1100@550
		(rectangle:  corner1 size: 10@10 at: 4700@900 )	
	   	(rectangle:  corner2 size: 10@10 at: 5825@1605 ) 
		"comment"
'! !

!GeoDSLModel methodsFor: 'opening' stamp: 'hk 8/18/2014 17:56'!
addMenuAndToolbarsTo: aDock 
	"a subclass may add much more"
	aDock
		add: 'Zoom' translated
		font: window theme menuBarFont
		icon: nil
		help: 'zoom operations' translated
		subMenu: self zoomMenu! !

!GeoDSLModel methodsFor: 'opening' stamp: 'hk 8/18/2014 16:10'!
decorateWindowFor: aGeoDSLDrawing 
	|  dock |
	dock := window newToolDockingBar.
	self addMenuAndToolbarsTo: dock .
	window
		addMorph: dock
		fullFrame: ((0 @ 0 corner: 1 @ 0) asLayoutFrame bottomOffset: dock minExtent y). 
	window addMorph: aGeoDSLDrawing canvas fullFrame: ((0@0 corner: 1@1) asLayoutFrame topOffset: dock minExtent y)

! !

!GeoDSLModel methodsFor: 'opening' stamp: 'hk 8/18/2014 16:11'!
decorateWindowFor: aGeoDSLDrawing With: aFileNameOrNil
	|file|
	self decorateWindowFor: aGeoDSLDrawing .
	[file := Form fromFileNamed: aFileNameOrNil .
	drawing canvas backgroundMorph: ((file) scaledToSize: aGeoDSLDrawing screenBounds extent) asMorph]
		on: Error do: [aFileNameOrNil ifNotNil: [GrowlMorph openWithLabel: 'no image on startup' contents: aFileNameOrNil] ].
	window themeChanged
	
! !

!GeoDSLModel methodsFor: 'accessing' stamp: 'hk 1/23/2014 14:41'!
drawing
	^ drawing! !

!GeoDSLModel methodsFor: 'initialize-release' stamp: 'hk 8/25/2014 17:59'!
drawingFrom: aSpec 
	GeoDSLCompiler drawingClass ifNil: 
		[GeoDSLCompiler drawingClass: (Smalltalk at: #GeoDSLDrawing)] .
	^drawing := GeoDSLCompiler compile: aSpec .
! !

!GeoDSLModel methodsFor: 'opening' stamp: 'hk 8/18/2014 17:20'!
open
	^self openWithBackground: nil 

! !

!GeoDSLModel methodsFor: 'opening' stamp: 'hk 8/18/2014 17:45'!
openWithBackground: aFileNameOrNil 

	window := (StandardWindow labelled: drawing title) model: self. 
	self decorateWindowFor: drawing With: aFileNameOrNil .
	window openInWorld;
			width: 10 + drawing screenBounds width ;
			height: drawing screenBounds height   .	
	^window ! !

!GeoDSLModel methodsFor: 'opening' stamp: 'hk 8/18/2014 17:55'!
zoomMenu
	|zoomMenu|
	(zoomMenu := window newMenu)
		addToggle: 'zoomIn' translated
		target: drawing
		selector: #zoomIn.
	zoomMenu lastItem
		font: window theme menuFont;
		keyText: 'Ctrl+I'.
	zoomMenu
		addToggle: 'zoomOut' translated
		target: drawing
		selector: #zoomOut
		getStateSelector: nil
		enablementSelector: nil.
	zoomMenu lastItem
		font: window theme menuFont;
		keyText: 'Ctrl+O'.
	zoomMenu
		addToggle: 'resetZoom' translated
		target: drawing
		selector: #resetZoom.
	zoomMenu lastItem
		font: window theme menuFont;
		keyText: 'Ctrl+R'.
	^zoomMenu! !

!GeoDSLModel methodsFor: 'opening' stamp: 'hk 2/22/2014 13:24'!
zoomToolbar
	^window newToolbar: 
				{window
					newButtonFor: drawing
					getState: nil
					action: #zoomIn
					arguments: nil
					getEnabled: nil
					label: 'zoom in'
					help: 'zoom in by a certain amount'. 
				window
					newButtonFor: drawing
					getState: nil
					action: #zoomOut
					arguments: nil
					getEnabled: nil
					label: 'zoom out'
					help: 'zoom out by a certain amount'.
				window
					newButtonFor: drawing
					getState: nil
					action: #resetZoom
					arguments: nil
					getEnabled: nil
					label: 'reset zoom'
					help: 'zoom to original'}

! !

GeoDSLModel subclass: #TrafficSimulation
	instanceVariableNames: 'cars road tick running wait'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GeoDSL-Examples'!

!TrafficSimulation commentStamp: '<historical>' prior: 0!
A TrafficSimulation is an example for the GeoDSL.
It demonstrates the traffic phase transition from free flow to congestion when the car density becomes too high for a certain velocity. 

Instance Variables
	cars:				<OrderedCollection of TSCar>
	road:				<TSRoad>
	tick:				<Integer>			- simulation tick distance in milliseconds
	wait:				<Integer>			- tick - wait = execution time
	running:			<Boolean>			- toggle											  
!

!TrafficSimulation class methodsFor: 'as yet unclassified' stamp: 'hk 1/28/2014 16:44'!
newOn: aTSRoad with: noOfCars
 	^(self new)
		road: aTSRoad;
		cars: noOfCars;
		open
		! !

!TrafficSimulation methodsFor: 'opening' stamp: 'hk 2/22/2014 14:56'!
addMenuAndToolbarsTo: aDock 
	super addMenuAndToolbarsTo: aDock .
	aDock addMorphBack: self simulationToolbar .
	aDock addMorphBack: self carToolbar .
! !

!TrafficSimulation methodsFor: 'accessing' stamp: 'hk 1/30/2014 15:15'!
carAt: index 
	"modulo - assuming a circular list"  
	^cars at: ((index - 1 + cars size) \\ cars size) + 1! !

!TrafficSimulation methodsFor: 'opening' stamp: 'hk 7/7/2014 18:30'!
carToolbar
 ^window newToolbar: { 
		window
			newButtonFor: self
			getState: nil
			action: #sleep
			arguments: nil
			getEnabled: nil
			label: 'sleep'
			help: 'drivers go to sleep'. 
		window
			newButtonFor: self
			getState: nil
			action: #dontSleep
			arguments: nil
			getEnabled: nil
			label: 'dont sleep'
			help: 'drivers awake'. 
		window
			newButtonFor: self
			getState: nil
			action: #stopFirstCar
			arguments: nil
			getEnabled: nil
			label: 'stop car 1'
			help: 'stops the first car'. 
		window
			newButtonFor: self
			getState: nil
			action: #letGoFirstCar
			arguments: nil
			getEnabled: nil
			label: 'go car 1'
			help: 'lets the first car go'. }
! !

!TrafficSimulation methodsFor: 'initialize-release' stamp: 'hk 1/31/2014 13:00'!
cars: aNumber 
	1 to: aNumber do: 
		[:i | |c|
		c := TSCar newOn: self.
		c name: 'c', i asString.
		c label: ''.
		c carIndex: i.
		cars addLast: c]! !

!TrafficSimulation methodsFor: 'simulation' stamp: 'hk 7/7/2014 18:27'!
dontSleep
	cars do: [ :e | e sleepTime: 0 ]! !

!TrafficSimulation methodsFor: 'initialize-release' stamp: 'hk 7/7/2014 18:43'!
initialize 
	cars := OrderedCollection new.
	tick := 30.
	wait := 10. 	
	running := false! !

!TrafficSimulation methodsFor: 'simulation' stamp: 'hk 5/22/2014 15:43'!
letGoFirstCar
	(cars at:  1) initializeVelocity ! !

!TrafficSimulation methodsFor: 'simulation' stamp: 'hk 1/30/2014 18:05'!
next
	cars do: [ :e | e next ]! !

!TrafficSimulation methodsFor: 'opening' stamp: 'hk 8/18/2014 16:22'!
open 
	| i |	
	self drawingFrom: GeoDSLExample geoSpecTrafficSimulation .
	self openWithBackground: 'XXX.jpg'. 
	drawing add:  road.
	i := 0.  
	cars reverseDo: [ :e | e location: (road roadPointAt: (i := i+1) ). e roadIndex: i. drawing add: e ]
! !

!TrafficSimulation methodsFor: 'accessing' stamp: 'hk 1/28/2014 15:56'!
road
	^ road! !

!TrafficSimulation methodsFor: 'accessing' stamp: 'hk 1/28/2014 16:06'!
road: aTSRoad 
	road := aTSRoad! !

!TrafficSimulation methodsFor: 'simulation' stamp: 'hk 7/7/2014 18:39'!
run
	running := true.
	[[running] whileTrue: 
		[self next. 
		 (Delay forMilliseconds: (wait max: 1)) wait ] 
	] forkAt: (Processor activePriority+1)
! !

!TrafficSimulation methodsFor: 'opening' stamp: 'hk 1/31/2014 13:34'!
simulationToolbar
 ^window newToolbar: 
		{window
			newButtonFor: self
			getState: nil
			action: #run
			arguments: nil
			getEnabled: nil
			label: 'run'
			help: 'start simulation'. 
		window
			newButtonFor: self
			getState: nil
			action: #stop
			arguments: nil
			getEnabled: nil
			label: 'stop'
			help: 'stops simulation'.
		window
			newButtonFor: self
			getState: nil
			action: #inspect
			arguments: nil
			getEnabled: nil
			label: 'inspect'
			help: 'open inspector'.  }
! !

!TrafficSimulation methodsFor: 'simulation' stamp: 'hk 1/31/2014 13:40'!
sleep
	cars do: [ :e | e sleepTime: 5000 ]! !

!TrafficSimulation methodsFor: 'simulation' stamp: 'hk 1/30/2014 18:03'!
stop
	running := false
! !

!TrafficSimulation methodsFor: 'simulation' stamp: 'hk 1/31/2014 13:12'!
stopFirstCar
	(cars at:  1) stop! !

!TrafficSimulation methodsFor: 'accessing' stamp: 'hk 1/28/2014 21:02'!
tick
	^ tick! !

Object subclass: #GeoDSLObject
	instanceVariableNames: 'morphClass name label color morph location size bearing screenLocation screenSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GeoDSL'!

!GeoDSLObject commentStamp: '<historical>' prior: 0!
A GeoDSLObject stores the real world information of an object in a GeoDSL application.
In addition it stores all information that is needed by the GeoDSL parser to create its morph.

Instance Variables
	label:				<String>			the displayed name of the object
	morphClass:		<ClassName>		for morph creation (GeoDSLObject>>asMorph)
	name:				<String>			for programmed reference
	location 			<Point>
	size 				<Point>			the extent 
	bearing 			<Number>			in Radians
	morph:				<Morph>
	screenLocation:	<Point>			center of bounds
	screenSize:		<Point>			extent of bounds
!

!GeoDSLObject methodsFor: 'private' stamp: 'hk 1/27/2014 13:01'!
asBasicMorph
	"answer the bare morph that I describe"
	|m|
	m := (Smalltalk at: morphClass) newBounds: (0@0 extent: screenSize ).
	m center: screenLocation.
	self attribute: m.
	^ m! !

!GeoDSLObject methodsFor: 'converting' stamp: 'hk 1/27/2014 13:12'!
asMorph
	^morph := self asTransformationMorph! !

!GeoDSLObject methodsFor: 'private' stamp: 'hk 1/27/2014 13:12'!
asTransformationMorph
	"wrap the bare morph by ist transformation"
	|m|
	m := TransformationMorph new asFlexOf: self asBasicMorph.
	m angle: bearing.
	m name: name.
	^ m! !

!GeoDSLObject methodsFor: 'private' stamp: 'hk 9/8/2014 14:56'!
attribute: aMorph
	aMorph name:  name.
	(Color registeredColorNames includes: color) ifTrue: [aMorph color: (Color perform: color)].
	aMorph addMorph: 
		(label asTextMorph position: aMorph position).
	(aMorph respondsTo: #model: ) ifTrue: [ aMorph model: self ]
	! !

!GeoDSLObject methodsFor: 'accessing' stamp: 'hk 2/13/2014 16:26'!
bearing
	^bearing ! !

!GeoDSLObject methodsFor: 'accessing' stamp: 'hk 1/21/2014 17:33'!
bearing: aNumber
	bearing := aNumber! !

!GeoDSLObject methodsFor: 'accessing' stamp: 'hk 7/7/2014 16:19'!
color:  aString
	color := aString asSymbol! !

!GeoDSLObject methodsFor: 'accessing' stamp: 'hk 1/19/2014 19:26'!
label: anObject
	label := anObject! !

!GeoDSLObject methodsFor: 'accessing' stamp: 'hk 1/29/2014 15:52'!
location
	^location! !

!GeoDSLObject methodsFor: 'accessing' stamp: 'hk 7/7/2014 12:32'!
location: aPoint
	location := aPoint asNonFractionalPoint! !

!GeoDSLObject methodsFor: 'accessing' stamp: 'hk 1/27/2014 17:40'!
morph
	^morph ! !

!GeoDSLObject methodsFor: 'parsing-interface' stamp: 'hk 3/5/2014 15:08'!
morphClass
	^morphClass! !

!GeoDSLObject methodsFor: 'parsing-interface' stamp: 'hk 1/19/2014 19:31'!
morphClass: aClassName
	morphClass := aClassName! !

!GeoDSLObject methodsFor: 'parsing-interface' stamp: 'hk 1/29/2014 12:57'!
moveBy: aNumber right: aRadian zoom: aZoom
	"move and turn differentially"
	| delta |
	delta := Point radius: aNumber theta: bearing negated.
	location := location + delta.
	bearing := bearing - aRadian.
	bearing := bearing \\ (2 * Float pi).			"keep bearing within 2 pi"
	morph position:  (morph position) + (delta * aZoom).
"	screenLocation := morph center.			not sure that we need this"
	morph angle: bearing

! !

!GeoDSLObject methodsFor: 'parsing-interface' stamp: 'hk 7/7/2014 12:50'!
moveTo: aPoint bearing: aRadian zoom: aZoom
	"move and turn absolutely"
	| delta |
	"move the object"
	delta := aPoint - location.
	location := aPoint.
	bearing := aRadian.
	"move the morph"
	morph angle: bearing.
	morph position:  (morph position) + ((delta * aZoom)) "rounded".

"	screenLocation := morph center.			not sure that we need this"
				
				

! !

!GeoDSLObject methodsFor: 'accessing' stamp: 'hk 1/20/2014 17:00'!
name1
	^name ! !

!GeoDSLObject methodsFor: 'accessing' stamp: 'hk 1/20/2014 13:57'!
name: aString
	name := aString! !

!GeoDSLObject methodsFor: 'zoom & translation' stamp: 'hk 1/21/2014 17:34'!
realBounds
	^ location - (size / 2) extent: size ! !

!GeoDSLObject methodsFor: 'accessing' stamp: 'hk 1/21/2014 17:33'!
size: aPoint
	size := aPoint! !

!GeoDSLObject methodsFor: 'zoom & translation' stamp: 'hk 1/27/2014 18:44'!
zoom: aNumber translate: aPoint 
	screenLocation := (location * aNumber + aPoint) rounded.
	screenSize := (size * aNumber) rounded.
	morph ifNotNil: [self zoomMorph]! !

!GeoDSLObject methodsFor: 'zoom & translation' stamp: 'hk 1/27/2014 13:18'!
zoomMorph
	morph bounds: (self asTransformationMorph bounds).
	morph center: screenLocation! !

GeoDSLObject subclass: #GeoDSLPathshape
	instanceVariableNames: 'points closed screenPoints'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GeoDSL'!

!GeoDSLPathshape commentStamp: '<historical>' prior: 0!
A GeoDSLPathshape represents polygons and polylines.

Instance Variables
	points:			<Array>		my real world points
	screenPoints:	<Array>
	closed:			<Boolean>		true for Polygon, false for Polyline


Todo: =========================================
- Tests für interface-agv!

!GeoDSLPathshape class methodsFor: 'instance creation' stamp: 'hk 2/13/2014 14:48'!
vertices: anArrayOfPoints 
	"self vertices: ( Array with: 0@0 with: 1@1 with: 2@2)"
	^ self vertices: anArrayOfPoints closed: false! !

!GeoDSLPathshape class methodsFor: 'instance creation' stamp: 'hk 2/13/2014 14:47'!
vertices: anArrayOfPoints closed: aBoolean
	"self vertices: ( Array with: 0@0 with: 1@1 with: 2@2)"
	^ self new initWithPoints: anArrayOfPoints closed: aBoolean! !

!GeoDSLPathshape methodsFor: 'converting' stamp: 'hk 1/27/2014 17:50'!
asBasicMorph
	"answer the bare morph that I describe"
	|m|
	m := PolygonMorph
				vertices: screenPoints
				color: Color transparent
				borderWidth: 1
				borderColor: Color black.
	closed ifTrue: [m makeClosed] ifFalse: [ m makeOpen  ].
	self attribute: m.
	^ m! !

!GeoDSLPathshape methodsFor: 'parsing-interface' stamp: 'hk 2/13/2014 14:51'!
closed
	^closed ! !

!GeoDSLPathshape methodsFor: 'parsing-interface' stamp: 'hk 1/19/2014 19:26'!
closed: aBoolean
	closed := aBoolean! !

!GeoDSLPathshape methodsFor: 'initialize-release' stamp: 'hk 2/13/2014 14:47'!
initWithPoints: anArrayOfPoints closed: aBoolean
	points := anArrayOfPoints.
	size := self realBounds extent.
	location := self realBounds  center.
	morphClass := PolygonMorph name asSymbol.
	closed := aBoolean! !

!GeoDSLPathshape methodsFor: 'interfave-agv' stamp: 'hk 7/4/2014 16:30'!
pointsWithin: aRadius from: aPoint

	^points select: [ :p |  (p - aPoint) r  < aRadius ]! !

!GeoDSLPathshape methodsFor: 'accessing' stamp: 'hk 1/20/2014 17:18'!
realBounds
	^(PathShape  vertices: points ) bounds! !

!GeoDSLPathshape methodsFor: 'zoom & translation' stamp: 'hk 1/23/2014 10:40'!
zoom: aNumber translate: aPoint 
	screenPoints := points collect: [ :e | e * aNumber + aPoint ].
	super zoom: aNumber translate: aPoint ! !

GeoDSLPathshape subclass: #TSRoad
	instanceVariableNames: 'roadMaxV'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GeoDSL-Examples'!

!TSRoad commentStamp: '<historical>' prior: 0!
A TSRoad is the road in the TrafficSimulation.

Instance Variables
	roadMaxV:		<Array of Number|nil>		- the maxV signs (same size as points)

!

!TSRoad class methodsFor: 'roads' stamp: 'hk 1/28/2014 17:07'!
pointsRoad1
	"self road1"  "almost an oval using 64 points"
 ^{	Point x: 5000 y: 1000 .
	Point x: 5050 y: 1000 .
	Point x: 5100 y: 1000 .
	Point x: 5150 y: 1000 .
	Point x: 5200 y: 1000 .
	Point x: 5250 y: 1000 .
	Point x: 5300 y: 1000 .
	Point x: 5350 y: 1000 .
	Point x: 5400 y: 1000 .
	Point x: 5450 y: 1000 .
	Point x: 5500 y: 1000 .
	Point x: 5550 y: 1000 .
	Point x: 5590 y: 1010 .
	Point x: 5630 y: 1020 .
	Point x: 5660 y: 1040 .
	Point x: 5690 y: 1060 .
	Point x: 5710 y: 1090 .
	Point x: 5730 y: 1120 .
	Point x: 5740 y: 1160 .
	Point x: 5750 y: 1200 .
	Point x: 5750 y: 1250 .
	Point x: 5750 y: 1300 .
	Point x: 5740 y: 1340 .
	Point x: 5730 y: 1380 .
	Point x: 5710 y: 1410 .
	Point x: 5690 y: 1440 .
	Point x: 5660 y: 1460 .
	Point x: 5630 y: 1480 .
	Point x: 5590 y: 1490 .
	Point x: 5550 y: 1500 .
	Point x: 5500 y: 1500 .
	Point x: 5450 y: 1500 .
	Point x: 5400 y: 1500 .
	Point x: 5350 y: 1500 .
	Point x: 5300 y: 1490 .
	Point x: 5250 y: 1480 .
	Point x: 5200 y: 1470 .
	Point x: 5150 y: 1460 .
	Point x: 5100 y: 1450 .
	Point x: 5050 y: 1440 .
	Point x: 5000 y: 1430 .
	Point x: 4960 y: 1420 .
	Point x: 4920 y: 1410 .
	Point x: 4890 y: 1400 .
	Point x: 4860 y: 1390 .
	Point x: 4840 y: 1380 .
	Point x: 4820 y: 1360 .
	Point x: 4810 y: 1340 .
	Point x: 4800 y: 1310 .
	Point x: 4800 y: 1290 .
	Point x: 4810 y: 1260 .
	Point x: 4820 y: 1230 .
	Point x: 4840 y: 1190 .
	Point x: 4860 y: 1150 .
	Point x: 4880 y: 1120 .
	Point x: 4900 y: 1090 .
	Point x: 4920 y: 1070 .
	Point x: 4930 y: 1050 .
	Point x: 4940 y: 1040 .
	Point x: 4950 y: 1030 .
	Point x: 4960 y: 1020 .
	Point x: 4970 y: 1012 .
	Point x: 4980 y: 1006 .
	Point x: 4990 y: 1002 
}! !

!TSRoad class methodsFor: 'instance creation' stamp: 'hk 2/13/2014 14:49'!
road1
	^(self new)
		initWithPoints:  self pointsRoad1 closed: true;
		initRoadPointsWithSpeedLimits: self speedLimits1 ;
		name: 'road1' 
		! !

!TSRoad class methodsFor: 'roads' stamp: 'hk 7/7/2014 18:24'!
speedLimits1
	"self speedLimits1"  "speed limits for road1 0 =  no limit"
 	| sl |
	sl := (IntegerArray ofSize: self pointsRoad1 size).
"	50 to: 63 do: [:i | sl at: i put:  10 ] .	""36 km/h"
	^sl
	! !

!TSRoad methodsFor: 'simulation' stamp: 'hk 1/30/2014 13:42'!
index: index behind: otherIndex 
	"i am only behind if otherIndex is not greater than half the road"
	^ (otherIndex - index + points size \\ points size) < (points size / 2)! !

!TSRoad methodsFor: 'initialize-release' stamp: 'hk 1/31/2014 12:23'!
initRoadPointsWithSpeedLimits: anArray 
	"converts the road points 'points' and adds the speedlimits"
	| nextPoint |
	anArray size = points size
		ifFalse: [self error: 'wrong size of ' , anArray printString].
	nextPoint := points first. 
	(1 to: points size) reverseDo: 
			[:i | | e b n |
			e := points at: i.
			b := ((nextPoint) - e) theta negated.
			nextPoint := e.
			n := RoadPoint
					from: e
					at: i
					bearing: b
					speedlimit: (anArray at: i).
			points at: i put: n]! !

!TSRoad methodsFor: 'simulation' stamp: 'hk 1/29/2014 14:49'!
initialPosition 
	^self roadPointAt: 1! !

!TSRoad methodsFor: 'initialize-release' stamp: 'hk 1/28/2014 17:11'!
initialize 
	"I must be initialized carefully since I am used directly in GeoDSLDrawing"
	closed := true.
	bearing := 0.
	label := ''.
	morphClass := #PolygonMorph! !

!TSRoad methodsFor: 'simulation' stamp: 'hk 1/30/2014 14:32'!
nextIndexOf: index 
	"modulo"
	^ (index \\ points size) + 1 ! !

!TSRoad methodsFor: 'simulation' stamp: 'hk 1/30/2014 11:43'!
nextPositionFor: distance From: aPoint at: index 

	| nextP nextI r |
	nextI := self nextIndexOf: index.
	nextP :=  points at: nextI. 
	r := (nextP - aPoint) r. 
	(r > distance ) ifTrue: [ ^self roadPoint: aPoint + ((nextP - aPoint) * distance / r) at: index].
	(r < distance ) ifTrue: [^self nextPositionFor: distance-r From: nextP at: nextI ] .
	(r = distance ) ifTrue: [ ^self roadPointAt: nextI ]
	! !

!TSRoad methodsFor: 'simulation' stamp: 'hk 1/30/2014 12:31'!
roadPoint: aPoint at: index 
	| nextIndex |
	nextIndex := self nextIndexOf: index.
	^RoadPoint from: aPoint at: index bearing:  ((points at: nextIndex) - aPoint ) theta negated! !

!TSRoad methodsFor: 'simulation' stamp: 'hk 1/31/2014 12:43'!
roadPointAt: index 
	^ points at: index
! !

GeoDSLObject subclass: #TSCar
	instanceVariableNames: 'a d maxV v ts carIndex roadIndex minDistance sleeptime timeToAwake'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GeoDSL-Examples'!

!TSCar commentStamp: '<historical>' prior: 0!
A TSCar is a car in the TrafficSimulation.

Instance Variables
	a:					<Number>			- acceleration in m/s^2
	d:					<Number>			- de-acceleration in m/s^2
	minDistance:		<Number>			- physical distance - not crashed
	maxV:				<Number>			- of the car
	ts:					<TrafficSimulation> 
	v:					<Number>			- actual velocity of the car
	sleeptime:			<Integer>			- in milliseconds
	carIndex:			<Integer>			- in the collection of cars
	roadIndex:			<Integer>			- in the road. 0 = offRoad
!

!TSCar class methodsFor: 'as yet unclassified' stamp: 'hk 1/28/2014 12:56'!
newOn: aTrafficSimulation 
	^(self new) ts: aTrafficSimulation ! !

!TSCar methodsFor: 'simulation' stamp: 'hk 2/11/2014 13:48'!
accelerate
	(timeToAwake  > 0 ) ifTrue: [ timeToAwake := timeToAwake - ts tick ] ifFalse:
	[v := maxV min: (v + (a * ts tick /1000)) asFloat ]! !

!TSCar methodsFor: 'simulation' stamp: 'hk 2/14/2014 16:09'!
adjustV
	"answer my new v. if possible i am accelerated"
	| fd |
	"too close ?"
	(fd := self frontDistance - minDistance ) < 0 ifTrue: [^ self crash].
	" too fast ?"
	(ts road roadPointAt: roadIndex) speedlimit < v ifTrue: [ ^ self decelerate].	
	"the distance rule with hysteresis"
	2 * v < fd  ifTrue: [ ^ self accelerate ].
	v < fd  ifFalse: [ ^ self decelerate ].
! !

!TSCar methodsFor: 'simulation' stamp: 'hk 1/30/2014 12:48'!
carInFront
	^ts carAt: carIndex - 1! !

!TSCar methodsFor: 'accessing' stamp: 'hk 1/29/2014 11:32'!
carIndex:  anInteger
	carIndex := anInteger! !

!TSCar methodsFor: 'simulation' stamp: 'hk 2/14/2014 14:47'!
crash
	v > 0 ifTrue: [GrowlMorph 
		openWithLabel: 'car no ', carIndex printString,  ' emergency braking' translated contents: ''].
	v  := 0.
	timeToAwake := sleeptime ! !

!TSCar methodsFor: 'simulation' stamp: 'hk 2/14/2014 15:24'!
decelerate
	v := (v + (d * ts tick /1000) ) max: 0.
	(v=0) ifTrue: [ timeToAwake := sleeptime ]! !

!TSCar methodsFor: 'simulation' stamp: 'hk 1/31/2014 07:04'!
distanceToGo
	"answer the distance to go with new v"
	self adjustV  .
	^v * ts tick / 1000
! !

!TSCar methodsFor: 'simulation' stamp: 'hk 2/14/2014 15:18'!
frontDistance
	"distance to car in front or > maxV. we neglect curves"
	(ts road index: roadIndex  behind: self carInFront roadIndex ) 
		ifTrue: [^(self carInFront location - location) r]
		ifFalse: [ ^2 * maxV+minDistance ]
! !

!TSCar methodsFor: 'initialize-release' stamp: 'hk 2/14/2014 14:05'!
initialize
	self initializeVelocities .
	sleeptime := 0.
	timeToAwake  := 0.
	minDistance := 55. "m"
	v := 0.	"m/s^2"
	roadIndex := 0.  "offRoad"
	"init super instance vars"
	morphClass := #GeoDSLCar.
	size := 50 @ 20.	"m - aber vergrößert"
	name := ''.
	label := ''.
	bearing := 0.! !

!TSCar methodsFor: 'initialize-release' stamp: 'hk 7/7/2014 18:20'!
initializeVelocities
	a := 4. 	"m/s^2"
	d := -16 .	"m/s^2"
	self initializeVelocity 
! !

!TSCar methodsFor: 'initialize-release' stamp: 'hk 7/7/2014 18:33'!
initializeVelocity
	maxV := 80.	"m/s"
! !

!TSCar methodsFor: 'simulation' stamp: 'hk 1/29/2014 17:57'!
moveTo: aRoadPoint
	self moveTo: aRoadPoint  bearing: aRoadPoint bearing zoom:  ts drawing zoom .  
	roadIndex := aRoadPoint index ! !

!TSCar methodsFor: 'simulation' stamp: 'hk 2/14/2014 14:26'!
next
	"advance me as far as can be - distanceToGo"  
	 self moveTo: (ts road nextPositionFor: self distanceToGo  From: location  at: roadIndex) ! !

!TSCar methodsFor: 'accessing' stamp: 'hk 1/30/2014 13:41'!
roadIndex
	^roadIndex! !

!TSCar methodsFor: 'accessing' stamp: 'hk 2/14/2014 14:16'!
roadIndex: i
	roadIndex := i! !

!TSCar methodsFor: 'accessing' stamp: 'hk 1/31/2014 06:47'!
sleepTime: milliSeconds
	sleeptime := milliSeconds 
	! !

!TSCar methodsFor: 'simulation' stamp: 'hk 5/22/2014 15:44'!
slowDown
	maxV := 20.	"m/s"
! !

!TSCar methodsFor: 'simulation' stamp: 'hk 1/31/2014 13:19'!
stop
	maxV  := 0! !

!TSCar methodsFor: 'accessing' stamp: 'hk 1/28/2014 12:56'!
ts: aTrafficSimulation
	ts := aTrafficSimulation! !
GeoDSLCar initialize!
